@@ -1,12 +1,13 @@
 #!/usr/bin/perl
 #
-# Copyright © 1996 Andy Guy <awpguy@acs.ucalgary.ca>
+# Copyright © 1996 Andy Guy <andy@cyteen.org>
 # Copyright © 1998 Martin Schulze <joey@infodrom.north.de>
 # Copyright © 1999, 2009 Raphaël Hertzog <hertzog@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 of the License.
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,8 +21,6 @@ use strict;
 use warnings;
 
 eval q{
-    pop @INC if $INC[-1] eq '.';
-
     use Dpkg; # Dummy import to require the presence of Dpkg::*.
 };
 if ($@) {
@@ -29,7 +28,8 @@ if ($@) {
     exit 1;
 }
 
-use Dselect::Ftp;
+use Dselect::Method;
+use Dselect::Method::Ftp;
 
 # deal with arguments
 my $vardir = $ARGV[0];
