@@ -1,4 +1,5 @@
 # Copyright © 2010-2011 Raphaël Hertzog <hertzog@debian.org>
+# Copyright © 2012-2022 Guillem Jover <guillem@debian.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,20 +14,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-package Dpkg::BuildFlags;
-
-use strict;
-use warnings;
-
-our $VERSION = '1.03';
-
-use Dpkg ();
-use Dpkg::Gettext;
-use Dpkg::Build::Env;
-use Dpkg::BuildOptions;
-use Dpkg::ErrorHandling;
-use Dpkg::Vendor qw(run_vendor_hook);
-
 =encoding utf8
 
 =head1 NAME
@@ -35,18 +22,42 @@ Dpkg::BuildFlags - query build flags
 
 =head1 DESCRIPTION
 
-The Dpkg::BuildFlags object is used by dpkg-buildflags and can be used
+This class is used by dpkg-buildflags and can be used
 to query the same information.
 
+=cut
+
+package Dpkg::BuildFlags 1.06;
+
+use strict;
+use warnings;
+
+use Dpkg ();
+use Dpkg::Gettext;
+use Dpkg::BuildEnv;
+use Dpkg::ErrorHandling;
+use Dpkg::Vendor qw(run_vendor_hook);
+
 =head1 METHODS
 
 =over 4
 
-=item $bf = Dpkg::BuildFlags->new()
+=item $bf = Dpkg::BuildFlags->new(%opts)
 
 Create a new Dpkg::BuildFlags object. It will be initialized based
 on the value of several configuration files and environment variables.
 
+Options:
+
+=over
+
+=item B<vendor_defaults>
+
+If set to false, then no vendor defaults are initialized
+(it defaults to true).
+
+=back
+
 =cut
 
 sub new {
@@ -56,10 +67,51 @@ sub new {
     my $self = {
     };
     bless $self, $class;
-    $self->load_vendor_defaults();
+
+    $opts{vendor_defaults} //= 1;
+
+    if ($opts{vendor_defaults}) {
+        $self->load_vendor_defaults();
+    } else {
+        $self->_init_vendor_defaults();
+    }
     return $self;
 }
 
+sub _init_vendor_defaults {
+    my $self = shift;
+
+    my @flags = qw(
+        ASFLAGS
+        ASFLAGS_FOR_BUILD
+        CPPFLAGS
+        CPPFLAGS_FOR_BUILD
+        CFLAGS
+        CFLAGS_FOR_BUILD
+        CXXFLAGS
+        CXXFLAGS_FOR_BUILD
+        OBJCFLAGS
+        OBJCFLAGS_FOR_BUILD
+        OBJCXXFLAGS
+        OBJCXXFLAGS_FOR_BUILD
+        DFLAGS
+        DFLAGS_FOR_BUILD
+        FFLAGS
+        FFLAGS_FOR_BUILD
+        FCFLAGS
+        FCFLAGS_FOR_BUILD
+        LDFLAGS
+        LDFLAGS_FOR_BUILD
+    );
+
+    $self->{features} = {};
+    $self->{builtins} = {};
+    $self->{optvals} = {};
+    $self->{flags} = { map { $_ => '' } @flags };
+    $self->{origin} = { map { $_ => 'vendor' } @flags };
+    $self->{maintainer} = { map { $_ => 0 } @flags };
+}
+
 =item $bf->load_vendor_defaults()
 
 Reset the flags stored to the default set provided by the vendor.
@@ -69,45 +121,8 @@ Reset the flags stored to the default set provided by the vendor.
 sub load_vendor_defaults {
     my $self = shift;
 
-    $self->{options} = {};
-    $self->{source} = {};
-    $self->{features} = {};
-    my $build_opts = Dpkg::BuildOptions->new();
-    $self->{build_options} = $build_opts;
-    my $default_flags = $build_opts->has('noopt') ? '-g -O0' : '-g -O2';
-    $self->{flags} = {
-	CPPFLAGS => '',
-	CFLAGS   => $default_flags,
-	CXXFLAGS => $default_flags,
-	OBJCFLAGS   => $default_flags,
-	OBJCXXFLAGS => $default_flags,
-	GCJFLAGS => $default_flags,
-	FFLAGS   => $default_flags,
-	FCFLAGS  => $default_flags,
-	LDFLAGS  => '',
-    };
-    $self->{origin} = {
-	CPPFLAGS => 'vendor',
-	CFLAGS   => 'vendor',
-	CXXFLAGS => 'vendor',
-	OBJCFLAGS   => 'vendor',
-	OBJCXXFLAGS => 'vendor',
-	GCJFLAGS => 'vendor',
-	FFLAGS   => 'vendor',
-	FCFLAGS  => 'vendor',
-	LDFLAGS  => 'vendor',
-    };
-    $self->{maintainer} = {
-	CPPFLAGS => 0,
-	CFLAGS   => 0,
-	CXXFLAGS => 0,
-	OBJCFLAGS   => 0,
-	OBJCXXFLAGS => 0,
-	GCJFLAGS => 0,
-	FFLAGS   => 0,
-	FCFLAGS  => 0,
-	LDFLAGS  => 0,
-    };
+    $self->_init_vendor_defaults();
+
     # The vendor hook will add the feature areas build flags.
     run_vendor_hook('update-buildflags', $self);
 }
@@ -143,7 +158,7 @@ sub load_user_config {
 =item $bf->load_environment_config()
 
 Update flags based on user directives stored in the environment. See
-dpkg-buildflags(1) for details.
+L<dpkg-buildflags(1)> for details.
 
 =cut
 
@@ -152,20 +167,20 @@ sub load_environment_config {
 
     foreach my $flag (keys %{$self->{flags}}) {
 	my $envvar = 'DEB_' . $flag . '_SET';
-	if (Dpkg::Build::Env::has($envvar)) {
-	    $self->set($flag, Dpkg::Build::Env::get($envvar), 'env');
+	if (Dpkg::BuildEnv::has($envvar)) {
+	    $self->set($flag, Dpkg::BuildEnv::get($envvar), 'env');
 	}
 	$envvar = 'DEB_' . $flag . '_STRIP';
-	if (Dpkg::Build::Env::has($envvar)) {
-	    $self->strip($flag, Dpkg::Build::Env::get($envvar), 'env');
+	if (Dpkg::BuildEnv::has($envvar)) {
+	    $self->strip($flag, Dpkg::BuildEnv::get($envvar), 'env');
 	}
 	$envvar = 'DEB_' . $flag . '_APPEND';
-	if (Dpkg::Build::Env::has($envvar)) {
-	    $self->append($flag, Dpkg::Build::Env::get($envvar), 'env');
+	if (Dpkg::BuildEnv::has($envvar)) {
+	    $self->append($flag, Dpkg::BuildEnv::get($envvar), 'env');
 	}
 	$envvar = 'DEB_' . $flag . '_PREPEND';
-	if (Dpkg::Build::Env::has($envvar)) {
-	    $self->prepend($flag, Dpkg::Build::Env::get($envvar), 'env');
+	if (Dpkg::BuildEnv::has($envvar)) {
+	    $self->prepend($flag, Dpkg::BuildEnv::get($envvar), 'env');
 	}
     }
 }
@@ -173,7 +188,7 @@ sub load_environment_config {
 =item $bf->load_maintainer_config()
 
 Update flags based on maintainer directives stored in the environment. See
-dpkg-buildflags(1) for details.
+L<dpkg-buildflags(1)> for details.
 
 =cut
 
@@ -182,20 +197,20 @@ sub load_maintainer_config {
 
     foreach my $flag (keys %{$self->{flags}}) {
 	my $envvar = 'DEB_' . $flag . '_MAINT_SET';
-	if (Dpkg::Build::Env::has($envvar)) {
-	    $self->set($flag, Dpkg::Build::Env::get($envvar), undef, 1);
+	if (Dpkg::BuildEnv::has($envvar)) {
+	    $self->set($flag, Dpkg::BuildEnv::get($envvar), undef, 1);
 	}
 	$envvar = 'DEB_' . $flag . '_MAINT_STRIP';
-	if (Dpkg::Build::Env::has($envvar)) {
-	    $self->strip($flag, Dpkg::Build::Env::get($envvar), undef, 1);
+	if (Dpkg::BuildEnv::has($envvar)) {
+	    $self->strip($flag, Dpkg::BuildEnv::get($envvar), undef, 1);
 	}
 	$envvar = 'DEB_' . $flag . '_MAINT_APPEND';
-	if (Dpkg::Build::Env::has($envvar)) {
-	    $self->append($flag, Dpkg::Build::Env::get($envvar), undef, 1);
+	if (Dpkg::BuildEnv::has($envvar)) {
+	    $self->append($flag, Dpkg::BuildEnv::get($envvar), undef, 1);
 	}
 	$envvar = 'DEB_' . $flag . '_MAINT_PREPEND';
-	if (Dpkg::Build::Env::has($envvar)) {
-	    $self->prepend($flag, Dpkg::Build::Env::get($envvar), undef, 1);
+	if (Dpkg::BuildEnv::has($envvar)) {
+	    $self->prepend($flag, Dpkg::BuildEnv::get($envvar), undef, 1);
 	}
     }
 }
@@ -218,6 +233,20 @@ sub load_config {
     $self->load_maintainer_config();
 }
 
+=item $bf->unset($flag)
+
+Unset the build flag $flag, so that it will not be known anymore.
+
+=cut
+
+sub unset {
+    my ($self, $flag) = @_;
+
+    delete $self->{flags}->{$flag};
+    delete $self->{origin}->{$flag};
+    delete $self->{maintainer}->{$flag};
+}
+
 =item $bf->set($flag, $value, $source, $maint)
 
 Update the build flag $flag with value $value and record its origin as
@@ -237,7 +266,7 @@ sub set {
 
 Update the boolean state of whether a specific feature within a known
 feature area has been enabled. The only currently known feature areas
-are "qa", "sanitize", "hardening" and "reproducible".
+are "future", "qa", "sanitize", "optimize", "hardening" and "reproducible".
 
 =cut
 
@@ -246,6 +275,94 @@ sub set_feature {
     $self->{features}{$area}{$feature} = $enabled;
 }
 
+=item $bf->get_feature($area, $feature)
+
+Returns the value for the given feature within a known feature area.
+This is relevant for builtin features where the feature has a ternary
+state of true, false and undef, and where the latter cannot be retrieved
+with use_feature().
+
+=cut
+
+sub get_feature {
+    my ($self, $area, $feature) = @_;
+
+    return if ! $self->has_features($area);
+    return $self->{features}{$area}{$feature};
+}
+
+=item $bf->use_feature($area, $feature)
+
+Returns true if the given feature within a known feature areas has been
+enabled, and false otherwise.
+The only currently recognized feature areas are "future", "qa", "sanitize",
+"optimize", "hardening" and "reproducible".
+
+=cut
+
+sub use_feature {
+    my ($self, $area, $feature) = @_;
+
+    return 0 if ! $self->has_features($area);
+    return 0 if ! $self->{features}{$area}{$feature};
+    return 1;
+}
+
+=item $bf->set_builtin($area, $feature, $enabled)
+
+Update the boolean state of whether a specific feature within a known
+feature area is handled (even if only in some architectures) as a builtin
+default by the compiler.
+
+=cut
+
+sub set_builtin {
+    my ($self, $area, $feature, $enabled) = @_;
+    $self->{builtins}{$area}{$feature} = $enabled;
+}
+
+=item $bf->get_builtins($area)
+
+Return, for the given area, a hash with keys as feature names, and values
+as booleans indicating whether the feature is handled as a builtin default
+by the compiler or not. Only features that might be handled as builtins on
+some architectures are returned as part of the hash. Missing features mean
+they are currently never handled as builtins by the compiler.
+
+=cut
+
+sub get_builtins {
+    my ($self, $area) = @_;
+    return if ! exists $self->{builtins}{$area};
+    return %{$self->{builtins}{$area}};
+}
+
+=item $bf->set_option_value($option, $value)
+
+B<Private> method to set the value of a build option.
+Do not use outside of the dpkg project.
+
+=cut
+
+sub set_option_value {
+    my ($self, $option, $value) = @_;
+
+    $self->{optvals}{$option} = $value;
+}
+
+=item $bf->get_option_value($option)
+
+B<Private> method to get the value of a build option.
+Do not use outside of the dpkg project.
+
+=cut
+
+sub get_option_value {
+    my ($self, $option) = @_;
+
+    return $self->{optvals}{$option};
+}
+
 =item $bf->strip($flag, $value, $source, $maint)
 
 Update the build flag $flag by stripping the flags listed in $value and
@@ -256,12 +373,12 @@ if $maint is defined and true.
 
 sub strip {
     my ($self, $flag, $value, $src, $maint) = @_;
-    foreach my $tostrip (split(/\s+/, $value)) {
-	next unless length $tostrip;
-	$self->{flags}->{$flag} =~ s/(^|\s+)\Q$tostrip\E(\s+|$)/ /g;
-    }
-    $self->{flags}->{$flag} =~ s/^\s+//g;
-    $self->{flags}->{$flag} =~ s/\s+$//g;
+
+    my %strip = map { $_ => 1 } split /\s+/, $value;
+
+    $self->{flags}->{$flag} = join q{ }, grep {
+        ! exists $strip{$_}
+    } split q{ }, $self->{flags}{$flag};
     $self->{origin}->{$flag} = $src if defined $src;
     $self->{maintainer}->{$flag} = $maint if $maint;
 }
@@ -308,7 +425,7 @@ sub prepend {
 =item $bf->update_from_conffile($file, $source)
 
 Update the current build flags based on the configuration directives
-contained in $file. See dpkg-buildflags(1) for the format of the directives.
+contained in $file. See L<dpkg-buildflags(1)> for the format of the directives.
 
 $source is the origin recorded for any build flag set or modified.
 
@@ -409,8 +526,8 @@ sub is_maintainer_modified {
 =item $bf->has_features($area)
 
 Returns true if the given area of features is known, and false otherwise.
-The only currently recognized feature areas are "qa", "sanitize", "hardening"
-and "reproducible".
+The only currently recognized feature areas are "future", "qa", "sanitize",
+"optimize", "hardening" and "reproducible".
 
 =cut
 
@@ -446,6 +563,21 @@ sub list {
 
 =head1 CHANGES
 
+=head2 Version 1.06 (dpkg 1.21.15)
+
+New method: $bf->get_feature().
+
+=head2 Version 1.05 (dpkg 1.21.14)
+
+New option: 'vendor_defaults' in new().
+
+New methods: $bf->load_vendor_defaults(), $bf->use_feature(),
+$bf->set_builtin(), $bf->get_builtins().
+
+=head2 Version 1.04 (dpkg 1.20.0)
+
+New method: $bf->unset().
+
 =head2 Version 1.03 (dpkg 1.16.5)
 
 New method: $bf->get_feature_areas() to list possible values for
